<style>
    body {
        background-color: {{body_background_color}};
        color: {{text_color}};
        font-family: Arial;
    }

    .headlinewrap{
        display: flex;
        flex-direction: row;
        align-content: center;
        align-items: center;
    }

    .brand {
        font-size: 0.7em;
        padding: 0.6em;
        background-color: gold;
        border-radius: 0.3em;
        color: black;
        margin-left: 1em;
        margin-bottom: 2em;
        font-weight:bold
    }
    
    .source-pre {
        font-family: Courier, monospace;
        white-space: -moz-pre-wrap; /* Mozilla, supported since 1999 */
        white-space: -pre-wrap; /* Opera */
        white-space: -o-pre-wrap; /* Opera */
        white-space: pre-wrap; /* CSS3 - Text module (Candidate Recommendation) http://www.w3.org/TR/css3-text/#white-space */
        word-wrap: break-word; /* IE 5.5+ */
        background-color: {{tag_pre_background_color}};
        color: {{tag_pre_text_color}};
        padding: 1.2em !important;
        font-size: 1.2em;
    }

    #headline {
        color: {{headline_color}};
    }
    .bk-toolbar {position: fixed;}

    .slick-header-column {
        background-color: {{datatable_header_color}} !important;
        background-image: none !important;
        font-size: 130%;
    }

    .slick-row.even {
        background-color: {{datatable_row_color_even}} !important;
    }

    .slick-row.odd {
        background-color: {{datatable_row_color_odd}} !important;
    }

    .bk-root .bk-bs-nav-tabs>li.bk-bs-active>span {
        background-color: {{tab_active_background_color}} !important;
        color: {{tab_active_color}} !important;
        border-color: {{tab_active_background_color}} !important;
    }

    .bk-root .bk-bs-nav>li>span:hover {
        background-color: {{tab_active_background_color}} !important;
        border-color: {{tab_active_background_color}} !important;
        color: {{tab_active_color}} !important;
    }

    .bk-tooltip {
        border-radius: 3px;
        background-color: {{tooltip_background_color}} !important;
        border-color: {{tooltip_background_color}} !important;
    }

    .bk-tooltip-row-label {
        color: {{tooltip_text_color_label}} !important;
    }

    .bk-tooltip-row-value {
        color: {{tooltip_text_color_value}} !important;
    }

    .label {
        font-weight: bold;
        line-height: 1.6em;
    }

    .table-title > div {
        display: block !important;
    }

    .table-title > div > .bk {
        font-weight: bold;
        font-size: 1.1em;
        line-height: 1.6em;
        white-space: nowrap;
        overflow: hidden;
        text-overflow: ellipsis;
    }

    .table-info {
        padding-bottom: .6em;
    }

    .panel-title > div {
        display: block !important;
    }

    .panel-title > div > .bk {
        font-weight: bold;
        font-size: 1.5em;
        line-height: 2.1em;
        white-space: nowrap;
        overflow: hidden;
        text-overflow: ellipsis;
    }

    .config-title > div {
        display: block !important;
        padding-top: 1em !important;
        padding-bottom: .6em !important;
    }

    .config-title > div > .bk {
        font-weight: bold;
        font-size: 1.1em;
        white-space: nowrap;
        overflow: hidden;
        text-overflow: ellipsis;
    }

    .bk-root .wrong-config{
        color:#d9534f;
        font-size: 1.1em;
        white-space: nowrap;
        overflow: hidden;
        text-overflow: ellipsis;
    }

    .footer {
        margin-top: 50px;
        text-align: right;
        padding-right: 1em;
        font-size: 0.8em;
    }
</style>
